/***************************************************************
 * Name:      reportdialog.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "dxdefs.h"
#include "reportdialog.h"
#include "model/dbmodel.h"
#include "utils/dxutils.h"

IMPLEMENT_DYNAMIC_CLASS(ReportDialog, wxDialog)

BEGIN_EVENT_TABLE(ReportDialog, wxDialog)
    EVT_BUTTON(wxID_OK, ReportDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, ReportDialog::OnCancel)
    EVT_TREE_SEL_CHANGED(ID_REPORTTASKS, ReportDialog::OnTasksChanged)
    EVT_DATE_CHANGED(ID_REPORTSTART, ReportDialog::OnDate)
    EVT_DATE_CHANGED(ID_REPORTEND, ReportDialog::OnDate)
END_EVENT_TABLE()

ReportDialog::ReportDialog(wxWindow *parent, bool csv)
: wxDialog(parent, wxID_ANY, _("New report")), m_csv(csv)
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

ReportDialog::ReportDialog(wxWindow *parent, int id, bool csv)
: wxDialog(parent, wxID_ANY, _("New report")), m_csv(csv)
{
    createControls();
    m_tasks->selectID(id);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void ReportDialog::createControls()
{
    wxDateTime now = wxDateTime::Now();
    wxDateTime prev = wxDateTime::Now().Add(wxDateSpan(0, -1));
    prev = wxDateTime(1, prev.GetMonth(), prev.GetYear());
    wxDateTime oldest = idbmodel->oldestDate();
    if(prev.IsEarlierThan(oldest)) oldest = prev;
    
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    
    wxFlexGridSizer *gridSizer = new wxFlexGridSizer(2, 5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Task:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_tasks = new Tree(this, ID_REPORTTASKS);
    m_tasks->updateData();
    gridSizer->Add(m_tasks, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));
    
    m_subtaskLabel = new wxStaticText(this, wxID_ANY, _("With subtasks:"));
    gridSizer->Add(m_subtaskLabel, wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_subtask = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    m_subtask->SetValue(true);
    gridSizer->Add(m_subtask, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));
    
    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Detailed:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_detailed = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    m_detailed->SetValue(true);
    gridSizer->Add(m_detailed, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));
    
    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("Start date:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_start = new wxDatePickerCtrl(this, ID_REPORTSTART, prev, wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_start->SetRange(oldest, now);
    gridSizer->Add(m_start, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));
    
    gridSizer->Add(new wxStaticText(this, wxID_ANY, _("End date:")), wxSizerFlags().Align(wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL).Border(wxLEFT | wxUP, 5));
    m_end = new wxDatePickerCtrl(this, ID_REPORTEND, wxDateTime(prev.GetLastMonthDay(prev.GetMonth(), prev.GetYear()).GetDay(), prev.GetMonth(), prev.GetYear()), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_end->SetRange(oldest, now);
    gridSizer->Add(m_end, wxSizerFlags().Expand().Align(wxALIGN_CENTER_VERTICAL).Border(wxUP, 5));
    
    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void ReportDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_csv) createCsv();
    else createHtml();
    EndModal(wxID_OK);
}

void ReportDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void ReportDialog::OnTasksChanged(wxTreeEvent& event)
{
    if(!m_subtaskLabel) return;
    if(!m_subtask) return;
    wxTreeItemId item = event.GetItem();
    if(!item) return;
    IdItemData *data = dynamic_cast<IdItemData*>(m_tasks->GetItemData(item));
    if(!data) return;
    if(idbmodel->hasSubtasks(data->id()))
    {
        m_subtaskLabel->Show();
        m_subtask->Show();
        m_subtask->SetValue(true);
    }
    else
    {
        m_subtaskLabel->Hide();
        m_subtask->Hide();
        m_subtask->SetValue(false);
    }
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
}

void ReportDialog::OnDate(wxDateEvent& event)
{
    if(event.GetId() == ID_REPORTSTART)
    {
        if(m_start->GetValue().IsLaterThan(m_end->GetValue()))
        {
            m_start->SetValue(m_end->GetValue());
        }
    }
    else
    {
        if(m_end->GetValue().IsEarlierThan(m_start->GetValue()))
        {
            m_end->SetValue(m_start->GetValue());
        }
    }
}

void ReportDialog::createCsv()
{
    wxFileDialog dlg(this, _("Report path"), wxEmptyString, wxEmptyString, _("CSV files (*.csv)|*.csv"),
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ";";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    int id = -1;
    if(m_tasks->GetFocusedItem().IsOk())
    {
        id = dynamic_cast<IdItemData*>(m_tasks->GetItemData(m_tasks->GetFocusedItem()))->id();
    }
    Task task = idbmodel->getTask(id);
    text << wxString::Format(_("Report from %s to %s"), m_start->GetValue().Format("%x"), m_end->GetValue().Format("%x")) << endl;
    if(m_subtask->GetValue())
    {
        if(m_detailed->GetValue())
        {
            int depth = idbmodel->treeDepth(id);
            int max = idbmodel->maxTreeDepth(id);
            text << dxutils::enquotes(_("Task"), delim) << delim;
            for(int i = 0; i < max-depth; i++)
            {
                text << delim;
            }
            text << dxutils::enquotes(_("Start"), delim) << delim << dxutils::enquotes(_("End"), delim) << delim
                <<dxutils::enquotes(_("Time"), delim) << endl;
            text << dxutils::enquotes(task.name(), delim) << endl;
            dxTimeArray times = idbmodel->timesForReport(id, m_start->GetValue(), m_end->GetValue());
            idbmodel->sortTime(times, DbModel::TIS_START, true);
            for(size_t i=0; i < times.GetCount(); i++)
            {
                for(int j = 0; j < max-depth; j++)
                {
                    text << delim;
                }
                text << delim << dxutils::enquotes(times[i].start().Format("%x %X"), delim) << delim
                        << dxutils::enquotes(times[i].end().Format("%x %X"), delim) << delim
                        << dxutils::enquotes(idbmodel->getTimeString(times[i].id()), delim) << endl;
            }
            wxArrayInt childs = m_tasks->childsID(id);
            for(size_t i=0; i < childs.GetCount(); i++)
            {
                int chdepth = idbmodel->treeDepth(childs[i]);
                for(int j = 0; j < chdepth-depth; j++)
                {
                    text << delim;
                }
                text << dxutils::enquotes(idbmodel->getTask(childs[i]).name(), delim) << endl;
                dxTimeArray times = idbmodel->timesForReport(childs[i], m_start->GetValue(), m_end->GetValue());
                idbmodel->sortTime(times, DbModel::TIS_START, true);
                for(size_t j=0; j < times.GetCount(); j++)
                {
                    for(int k = 0; k < max-depth; k++)
                    {
                        text << delim;
                    }
                    text << delim << dxutils::enquotes(times[j].start().Format("%x %X"), delim) << delim
                            << dxutils::enquotes(times[j].end().Format("%x %X"), delim) << delim
                            << dxutils::enquotes(idbmodel->getTimeString(times[j].id()), delim) << endl;
                }
            }
            for(int i = 0; i < max-depth; i++)
            {
                text << delim;
            }
            text << delim << delim << dxutils::enquotes(_("Total"), delim) << delim
                    << idbmodel->timeSpanForReport(id, m_start->GetValue(), m_end->GetValue(), true).Format("%D:%H:%M") << endl;
        }
        else
        {
            int depth = idbmodel->treeDepth(id);
            int max = idbmodel->maxTreeDepth(id);
            text << dxutils::enquotes(_("Task"), delim) << delim;
            for(int i = 0; i < max-depth; i++)
            {
                text << delim;
            }
            text << dxutils::enquotes(_("Time"), delim) << endl;
            text << dxutils::enquotes(task.name(), delim) << delim;
            for(int i = 0; i < max-depth; i++)
            {
                text << delim;
            }
            text << idbmodel->timeSpanForReport(id, m_start->GetValue(), m_end->GetValue(), false).Format("%D:%H:%M") << endl;
            wxArrayInt childs = m_tasks->childsID(id);
            for(size_t i=0; i < childs.GetCount(); i++)
            {
                int chdepth = idbmodel->treeDepth(childs[i]);
                for(int j = 0; j < chdepth-depth; j++)
                {
                    text << delim;
                }
                text << dxutils::enquotes(idbmodel->getTask(childs[i]).name(), delim) << delim;
                for(int j = 0; j < max-chdepth; j++)
                {
                    text << delim;
                }
                text << idbmodel->timeSpanForReport(childs[i], m_start->GetValue(), m_end->GetValue(), false).Format("%D:%H:%M") << endl;
            }
            for(int i = 0; i < max-depth; i++)
            {
                text << delim;
            }
            text << dxutils::enquotes(_("Total"), delim) << delim
                    << idbmodel->timeSpanForReport(id, m_start->GetValue(), m_end->GetValue(), true).Format("%D:%H:%M") << endl;
        }
    }
    else
    {
        if(m_detailed->GetValue())
        {
            text << dxutils::enquotes(_("Task"), delim) << delim << dxutils::enquotes(_("Start"), delim) << delim 
                    << dxutils::enquotes(_("End"), delim) << delim << dxutils::enquotes(_("Time"), delim) << endl;
            text << dxutils::enquotes(task.name(), delim) << delim << delim << delim << endl;
            dxTimeArray times = idbmodel->timesForReport(id, m_start->GetValue(), m_end->GetValue());
            idbmodel->sortTime(times, DbModel::TIS_START, true);
            for(size_t i=0; i < times.GetCount(); i++)
            {
                text << delim << dxutils::enquotes(times[i].start().Format("%x %X"), delim) << delim
                        << dxutils::enquotes(times[i].end().Format("%x %X"), delim) << delim
                        << dxutils::enquotes(idbmodel->getTimeString(times[i].id()), delim) << endl;
            }
            text << delim << delim << dxutils::enquotes(_("Total"), delim) << delim
                    << idbmodel->timeSpanForReport(id, m_start->GetValue(), m_end->GetValue(), false).Format("%D:%H:%M") << endl;
        }
        else
        {
            text << dxutils::enquotes(_("Task"), delim) << delim << dxutils::enquotes(_("Time"), delim) << endl;
            text << dxutils::enquotes(task.name(), delim) << delim
                    << idbmodel->timeSpanForReport(id, m_start->GetValue(), m_end->GetValue(), false).Format("%D:%H:%M") << endl;
        }
    }
}

void ReportDialog::createHtml()
{
    wxFileDialog dlg(this, _("Report path"), wxEmptyString, wxEmptyString, _("HTML files (*.html)|*.html"),
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("html", path);
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    int id = -1;
    if(m_tasks->GetFocusedItem().IsOk())
    {
        id = dynamic_cast<IdItemData*>(m_tasks->GetItemData(m_tasks->GetFocusedItem()))->id();
    }
    Task task = idbmodel->getTask(id);
    text << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">" << endl;
    text << "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">" << endl;
    text << "<head>" << endl;
    text << "<title>" << wxString::Format(_("Report from %s to %s"), m_start->GetValue().Format("%x"), m_end->GetValue().Format("%x"))
        << "</title>" << endl;
    text << "<meta http-equiv=\"content-type\" content=\"text/html;charset=utf-8\" />" << endl;
    text << "<meta name=\"generator\" content=\"dxtime\" />" << endl;
    text << "</head>" << endl;
    text << "<body>" << endl;
    text << "<b>" << wxString::Format(_("Report from %s to %s"), m_start->GetValue().Format("%x"), m_end->GetValue().Format("%x")) << "</b><br><br>"
            << endl;
    if(m_subtask->GetValue())
    {
        if(m_detailed->GetValue())
        {
            int depth = idbmodel->treeDepth(id);
            int max = idbmodel->maxTreeDepth(id);
            text << "<table cellpadding=\"2\" cellspacing=\"0\">" << endl;
            text << "<tr>" << endl;
            text << "<td style=\"border-bottom: 2px solid\">" << _("Task") << "</td>";
            for(int i = 0; i < max-depth; i++)
            {
                text << "<td style=\"border-bottom: 2px solid\"></td>";
            }
            text << "<td style=\"border-bottom: 2px solid\">" << _("Start") << "</td>"
                    << "<td style=\"border-bottom: 2px solid\">" << _("End") << "</td>"
                    << "<td style=\"border-bottom: 2px solid\">" << _("Time") << "</td>" << endl;
            text << "</tr>" << endl;
            text << "<tr>" << endl;
            text << "<td>" << task.name() << "</td>";
            for(int i = 0; i < max-depth; i++)
            {
                text << "<td></td>";
            }
            text << "<td></td><td></td><td></td>" << endl;
            text << "</tr>" << endl;
            dxTimeArray times = idbmodel->timesForReport(id, m_start->GetValue(), m_end->GetValue());
            idbmodel->sortTime(times, DbModel::TIS_START, true);
            for(size_t i=0; i < times.GetCount(); i++)
            {
                text << "<tr>" << endl;
                for(int j = 0; j < max-depth+1; j++)
                {
                    text << "<td></td>";
                }
                text << "<td>" << times[i].start().Format("%x %X") << "</td>"
                        << "<td>" << times[i].end().Format("%x %X") << "</td>"
                        << "<td align=\"RIGHT\">" << idbmodel->getTimeString(times[i].id()) << "</td>" << endl;
                text << "</tr>" << endl;
            }
            wxArrayInt childs = m_tasks->childsID(id);
            for(size_t i=0; i < childs.GetCount(); i++)
            {
                text << "<tr>" << endl;
                int chdepth = idbmodel->treeDepth(childs[i]);
                for(int j = 0; j < chdepth-depth; j++)
                {
                    text << "<td></td>";
                }
                text << "<td>" << idbmodel->getTask(childs[i]).name() << "</td>";
                for(int j = 0; j < max-chdepth; j++)
                {
                    text << "<td></td>";
                }
                text << "<td></td><td></td><td></td>" << endl;
                text << "</tr>" << endl;
                dxTimeArray times = idbmodel->timesForReport(childs[i], m_start->GetValue(), m_end->GetValue());
                idbmodel->sortTime(times, DbModel::TIS_START, true);
                for(size_t j=0; j < times.GetCount(); j++)
                {
                    text << "<tr>" << endl;
                    for(int k = 0; k < max-depth+1; k++)
                    {
                        text << "<td></td>";
                    }
                    text << "<td>" << times[j].start().Format("%x %X") << "</td>"
                            << "<td>" << times[j].end().Format("%x %X") << "</td>"
                            << "<td align=\"RIGHT\">" << idbmodel->getTimeString(times[j].id()) << "</td>" << endl;
                    text << "</tr>" << endl;
                }
            }
            text << "<tr>" << endl;
            for(int i = 0; i < max-depth; i++)
            {
                text << "<td style=\"border-top: 2px solid\"></td>";
            }
            text << "<td style=\"border-top: 2px solid\"></td>" << "<td style=\"border-top: 2px solid\"></td>"
                    << "<td style=\"border-top: 2px solid\">" << _("Total") << "</td>"
                    << "<td style=\"border-top: 2px solid\" align=\"RIGHT\">"
                    << idbmodel->timeSpanForReport(id, m_start->GetValue(), m_end->GetValue(), true).Format("%D:%H:%M") << "</td>" << endl;
            text << "</tr>" << endl;
            text << "</table>" << endl;
        }
        else
        {
            int depth = idbmodel->treeDepth(id);
            int max = idbmodel->maxTreeDepth(id);
            text << "<table cellpadding=\"2\" cellspacing=\"0\">" << endl;
            text << "<tr>" << endl;
            text << "<td style=\"border-bottom: 2px solid\">" << _("Task") << "</td>";
            for(int i = 0; i < max-depth; i++)
            {
                text << "<td style=\"border-bottom: 2px solid\"></td>";
            }
            text << "<td style=\"border-bottom: 2px solid\">" << _("Time") << "</td>" << endl;
            text << "</tr>" << endl;
            text << "<tr>" << endl;
            text << "<td>" << task.name() << "</td>";
            for(int i = 0; i < max-depth; i++)
            {
                text << "<td></td>";
            }
            text << "<td align=\"RIGHT\">" << idbmodel->timeSpanForReport(id, m_start->GetValue(), m_end->GetValue(), false).Format("%D:%H:%M") << "</td>" << endl;
            text << "</tr>" << endl;
            wxArrayInt childs = m_tasks->childsID(id);
            for(size_t i=0; i < childs.GetCount(); i++)
            {
                text << "<tr>" << endl;
                int chdepth = idbmodel->treeDepth(childs[i]);
                for(int j = 0; j < chdepth-depth; j++)
                {
                    text << "<td></td>";
                }
                text << "<td>" << idbmodel->getTask(childs[i]).name() << "</td>";
                for(int j = 0; j < max-chdepth; j++)
                {
                    text << "<td></td>";
                }
                text << "<td align=\"RIGHT\">" << idbmodel->timeSpanForReport(childs[i], m_start->GetValue(), m_end->GetValue(), false).Format("%D:%H:%M") << "</td>" << endl;
                text << "</tr>" << endl;
            }
            text << "<tr>" << endl;
            for(int i = 0; i < max-depth; i++)
            {
                text << "<td style=\"border-top: 2px solid\"></td>";
            }
            text << "<td style=\"border-top: 2px solid\">" << _("Total") << "</td>"
                    << "<td style=\"border-top: 2px solid\" align=\"RIGHT\">"
                    << idbmodel->timeSpanForReport(id, m_start->GetValue(), m_end->GetValue(), true).Format("%D:%H:%M") << "</td>" << endl;
            text << "</tr>" << endl;
            text << "</table>" << endl;
        }
    }
    else
    {
        if(m_detailed->GetValue())
        {
            text << "<table cellpadding=\"2\" cellspacing=\"0\">" << endl;
            text << "<tr>" << endl;
            text << "<td style=\"border-bottom: 2px solid\">" << _("Task") << "</td>"
                    << "<td style=\"border-bottom: 2px solid\">" << _("Start") << "</td>" 
                    << "<td style=\"border-bottom: 2px solid\">" << _("End") << "</td>"
                    << "<td style=\"border-bottom: 2px solid\">" << _("Time") << "</td>" << endl;
            text << "</tr>" << endl;
            text << "<tr>" << endl;
            text << "<td>" << task.name() << "</td>" << "<td></td>" << "<td></td>" << "<td></td>" << endl;
            text << "</tr>" << endl;            
            dxTimeArray times = idbmodel->timesForReport(id, m_start->GetValue(), m_end->GetValue());
            idbmodel->sortTime(times, DbModel::TIS_START, true);
            for(size_t i=0; i < times.GetCount(); i++)
            {
                text << "<tr>" << endl;
                text << "<td></td>" << "<td>" << times[i].start().Format("%x %X") << "</td>"
                        << "<td>" << times[i].end().Format("%x %X") << "</td>"
                        << "<td align=\"RIGHT\">" << idbmodel->getTimeString(times[i].id()) << "</td>";
                text << "</tr>" << endl;
            }
            text << "<tr>" << endl;
            text << "<td style=\"border-top: 2px solid\"></td>" << "<td style=\"border-top: 2px solid\"></td>"
                    << "<td style=\"border-top: 2px solid\">" << _("Total") << "</td>"
                    << "<td style=\"border-top: 2px solid\" align=\"RIGHT\">"
                    << idbmodel->timeSpanForReport(id, m_start->GetValue(), m_end->GetValue(), false).Format("%D:%H:%M") << "</td>" << endl;
            text << "</tr>" << endl;
            text << "</table>" << endl;
        }
        else
        {
            text << "<table cellpadding=\"2\" cellspacing=\"0\">" << endl;
            text << "<tr>" << endl;
            text << "<td style=\"border-bottom: 2px solid\">" << _("Task") << "</td>"
                    << "<td style=\"border-bottom: 2px solid\">" << _("Time") << "</td>" << endl;
            text << "</tr>" << endl;
            text << "<tr>" << endl;
            text << "<td>" << task.name() << "</td>" << "<td align=\"RIGHT\">"
                    << idbmodel->timeSpanForReport(id, m_start->GetValue(), m_end->GetValue(), false).Format("%D:%H:%M") << "</td>" << endl;
            text << "</tr>" << endl;
            text << "</table>" << endl;
        }
    }
    text << "</body>" << endl;
    text << "</html>" << endl;
}
