/***************************************************************
 * Name:      dbmodel.cpp
 * Author:    David Vachulka (arch_dvx@users.sourceforge.net)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/stdpaths.h>
#include <wx/log.h>
#include <wx/arrimpl.cpp>
#include <wx/tokenzr.h>
#include "dbmodel.h"
#include "dbsort.h"
#include "sqlitesql.h"
#include "utils/settings.h"
#include "utils/dxutils.h"
#include "dxdefs.h"

WX_DEFINE_OBJARRAY(dxTaskArray)
WX_DEFINE_OBJARRAY(dxTimeArray)

wxDEFINE_EVENT(MODEL_UPDATED, wxCommandEvent);
wxDEFINE_EVENT(MODEL_MSG, wxCommandEvent);

DbModel *DbModel::m_this = NULL;

DbModel::DbModel() : m_parent(NULL)
{
}

DbModel::DbModel(wxWindow *parent) : m_parent(parent)
{
    m_this = this;
}

DbModel::~DbModel()
{
    if(!m_db.IsOpen())
        return;
    try
    {
        m_db.Close();
    }
    catch(const wxSQLite3Exception& e)
    {
        wxLogError(wxString::Format("Close error: %s", e.GetMessage().c_str()));
    }
    m_this = NULL;
}

DbModel *DbModel::instance()
{
    return m_this;
}

void DbModel::initModel()
{
    wxSQLite3Database::InitializeSQLite();
}

void DbModel::shutdownModel()
{
    wxSQLite3Database::ShutdownSQLite();
}

wxString DbModel::getVersion()
{
    wxString version = wxSQLite3Database::GetWrapperVersion() << "\n"
                        << "SQLite3 " << wxSQLite3Database::GetVersion();
    return version;
}

bool DbModel::openDatabase()
{
    wxString path = wxStandardPaths::Get().GetUserDataDir();
    if(!wxDirExists(path))
    {
        wxMkdir(path);
    }
    path << wxFILE_SEP_PATH << settings.dbname();
    int err = SQLITE_OK;
    wxString errStr = wxEmptyString;
    try
    {
        m_db.Open(path);
    }
    catch(const wxSQLite3Exception &e)
    {
        err = e.GetErrorCode();
        errStr << e.GetMessage();
    }
    if(err == SQLITE_OK)
    {
        m_db.EnableForeignKeySupport(true);
        //Try if db selectable
        try
        {
            m_db.ExecuteScalar("SELECT count(*) FROM sqlite_master WHERE type='table'");
        }
        catch(const wxSQLite3Exception &e)
        {
            err = e.GetExtendedErrorCode();
            errStr << e.GetMessage();
        }
        if(!m_db.TableExists("version"))
            createDatabase();
        return true;
    }
    if(!errStr.empty())
        wxLogDebug(errStr);
    return false;
}

bool DbModel::closeDatabase()
{
    if(!m_db.IsOpen()) return true;
    try
    {
        m_db.Close();
    }
    catch(const wxSQLite3Exception& e)
    {
        wxLogDebug(wxString::Format("Close error: %s", e.GetMessage()));
        return false;
    }
    return true;
}

int DbModel::getLastId(const wxString &table)
{
    if(m_db.TableExists(table))
    {
        return m_db.ExecuteScalar(wxString::Format("SELECT max(id) FROM %s", table.c_str()))+1;
    }
    return 1;
}

//fill data arrays values
void DbModel::fillData()
{
    fillTasks();
    fillTimes();
    verifyTasks();
    verifyTimes();
}

void DbModel::fillTasks()
{
    if(!m_db.TableExists("task")) createDatabaseTable(TB_TASK);
}

void DbModel::fillTimes()
{
    if(!m_db.TableExists("time")) createDatabaseTable(TB_TIME);
}

void DbModel::createDatabase()
{
    for(int i =0; i < TB_END; i++)
    {
        createDatabaseTable((Tables)i);
    }
}

void DbModel::createDatabaseTable(DbModel::Tables table)
{
    switch(table){
    case TB_VERSION:
    {
        int i = 0;
        while(tb_version[i] != NULL)
        {
            try
            {
                m_db.ExecuteUpdate(tb_version[i]);
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogError(wxString::Format("Chyba v createDatabaseTable: %s", e.GetMessage().c_str()));
            }
            ++i;
        }
        break;
    }
    case TB_TASK:
    {
        int i = 0;
        while(tb_task[i] != NULL)
        {
            try
            {
                m_db.ExecuteUpdate(tb_task[i]);
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogError(wxString::Format("Chyba v createDatabaseTable: %s", e.GetMessage().c_str()));
            }
            ++i;
        }
        break;
    }
    case TB_TIME:
    {
        int i = 0;
        while(tb_time[i] != NULL)
        {
            try
            {
                m_db.ExecuteUpdate(tb_time[i]);
            }
            catch(const wxSQLite3Exception& e)
            {
                wxLogError(wxString::Format("Chyba v createDatabaseTable: %s", e.GetMessage().c_str()));
            }
            ++i;
        }
        break;
    }
    default:
        return;
    }
}

void DbModel::fireUpdated()
{
    wxCommandEvent event(MODEL_UPDATED);
    wxPostEvent(m_parent, event);
}

void DbModel::fireMsg(const wxString &msg)
{
    wxCommandEvent event(MODEL_MSG);
    event.SetString(msg);
    wxPostEvent(m_parent, event);
}

dxTaskArray DbModel::tasks()
{
    dxTaskArray ts;
    wxSQLite3Table tb = m_db.GetTable("SELECT * FROM task");
    for(int i=0; i < tb.GetRowCount(); i++)
    {
        tb.SetRow(i);
        ts.Add(Task(tb.GetInt("id"), tb.GetString("name"), tb.GetString("description"), tb.GetString("tree")));
    }
    if(!hasTask(-1))
    {
        addTask(Task(-1, _("Tasks"), wxEmptyString, wxEmptyString));
    }
    sortTask(ts, DbModel::TAS_TREE, true);
    return ts;
}

void DbModel::addTask(const Task &task)
{
    if(!hasTask(task.id()))
    {
        try
        {
            wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO task(id,name,description,tree) "
                                                          "VALUES(?,?,?,?)");
            st.Bind(1, task.id());
            st.Bind(2, task.name());
            st.Bind(3, task.description());
            st.Bind(4, task.tree());
            st.ExecuteUpdate();
            st.Finalize();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogError(wxString::Format("Error in addTask: %s", e.GetMessage().c_str()));
        }
    }
}

void DbModel::editTask(const Task &task)
{
    if(hasTask(task.id()))
    {
        Task ts = getTask(task.id());
        ts.setName(task.name());
        ts.setTree(task.tree());
        ts.setDescription(task.description());
        try
        {
            wxSQLite3Statement st = m_db.PrepareStatement("UPDATE task SET name=?,description=?,tree=?"
                                                          " WHERE id=?");
            st.Bind(1, task.name());
            st.Bind(2, task.description());
            st.Bind(3, task.tree());
            st.Bind(4, task.id());
            st.ExecuteUpdate();
            st.Finalize();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogError(wxString::Format("Error in editTask: %s", e.GetMessage().c_str()));
        }
    }
}

Task DbModel::getTask(int id)
{
    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM task WHERE id=%d", id));
    while(rs.NextRow())
    {
        return Task(rs.GetInt("id"), rs.GetString("name"), rs.GetString("description"), rs.GetString("tree"));
    }
    return Task();
}

bool DbModel::deleteTask(int id)
{
    if(hasSubtasks(id))
    {
        dxTaskArray subtasks = subTasks(id);
        for(size_t j=0; j < subtasks.GetCount(); j++)
        {
            deleteTask(subtasks[j].id());
        }
    }
    try
    {
        m_db.ExecuteUpdate(wxString::Format("DELETE FROM task WHERE id=%d", id));
    }
    catch(const wxSQLite3Exception& e)
    {
        wxLogError(wxString::Format("Error in deleteTask: %s", e.GetMessage().c_str()));
        return false;
    }
    return true;
}

bool DbModel::hasTask(int id)
{
    return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM task WHERE id=%d", id));
}

bool DbModel::hasSubtasks(int id)
{
    if(id == -1)
    {
        return m_db.ExecuteScalar("SELECT count(*) FROM task")>1;
    }
    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT tree FROM task");
    while(rs.NextRow())
    {
        wxArrayString strings = wxStringTokenize(rs.GetString(0), ".");
        for(size_t j=0; j < strings.GetCount(); j++)
        {
            if(strings[j].IsEmpty())
            {
                continue;
            }
            if(wxAtoi(strings[j]) == id)
            {
                return true;
            }
        }
    }
    return false;
}

dxTaskArray DbModel::subTasks(int id)
{
    dxTaskArray tasks;
    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM task");
    while(rs.NextRow())
    {
        wxArrayString strings = wxStringTokenize(rs.GetString("tree"), ".");
        for(size_t j=0; j < strings.GetCount(); j++)
        {
            if(strings[j].IsEmpty())
            {
                continue;
            }
            if(wxAtoi(strings[j]) == id)
            {
                tasks.Add(Task(rs.GetInt("id"), rs.GetString("name"), rs.GetString("description"), rs.GetString("tree")));
            }
        }
    }
    return tasks;
}

void DbModel::verifyTasks()
{
    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT id FROM task");
    while(rs.NextRow())
    {
        if(rs.GetInt(0) == -1)
        {
            continue;
        }
        else
        {
            if(!treeExists(rs.GetInt(0)))
            {
                deleteTask(rs.GetInt(0));
            }
        }
    }
    rs.Finalize();
}

bool DbModel::treeExists(int id)
{
    if(id == -1)
    {
        return true;
    }
    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT tree FROM task WHERE id=%d", id));
    while(rs.NextRow())
    {
        wxArrayString strings = wxStringTokenize(rs.GetString(0), ".");
        for(size_t j=0; j < strings.GetCount(); j++)
        {
            if(strings[j].IsEmpty())
            {
                continue;
            }
            if(!hasTask(wxAtoi(strings[j])))
            {
                return false;
            }
            if(wxAtoi(strings[j]) == id)
            {
                return false;
            }
        }
        return true;
    }
    rs.Finalize();
    return false;
}

int DbModel::treeDepth(int id)
{
    if(id == -1)
    {
        return 0;
    }
    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT tree FROM task WHERE id=%d", id));
    while(rs.NextRow())
    {
        return dxutils::numberOfChar('.', rs.GetString(0));
    }
    rs.Finalize();
    return -1;
}

int DbModel::maxTreeDepth(int id)
{
    if(id == -1)
    {
        int result = 0;
        wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT tree FROM task");
        while(rs.NextRow())
        {
            if(dxutils::numberOfChar('.', rs.GetString(0)) > result)
            {
                result = dxutils::numberOfChar('.', rs.GetString(0));
            }
        }
        rs.Finalize();
        return result;
    }
    int depth = 0;
    if(hasSubtasks(id))
    {
        dxTaskArray subtasks = subTasks(id);
        for(size_t j=0; j < subtasks.GetCount(); j++)
        {
            if(dxutils::numberOfChar('.', subtasks[j].tree()) > depth)
            {
                depth = dxutils::numberOfChar('.', subtasks[j].tree());
            }
        }
        return depth;
    }
    else
    {
        return 0;
    }
}

wxString DbModel::taskName(int id)
{
    if(id == -1) return wxEmptyString;
    return m_db.ExecuteQuery(wxString::Format("SELECT name FROM task WHERE id=%d", id)).GetString(0);
}

dxTimeArray DbModel::times()
{
    dxTimeArray ta;
    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT * FROM time");
    while(rs.NextRow())
    {
        ta.Add(Time(rs.GetInt("id"), rs.GetDateTime("start"), rs.GetDateTime("end"), rs.GetInt("taskid")));
    }
    rs.Finalize();
    return ta;
}

void DbModel::addTime(const Time &time)
{
    if(!hasTime(time.id()))
    {
        try
        {
            wxSQLite3Statement st = m_db.PrepareStatement("INSERT INTO time(id,start,end,taskid) "
                                                          "VALUES(?,?,?,?)");
            st.Bind(1, time.id());
            st.BindDateTime(2, time.start());
            st.BindDateTime(3, time.end());
            st.Bind(4, time.taskid());
            st.ExecuteUpdate();
            st.Finalize();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogError(wxString::Format("Error in addTime: %s", e.GetMessage().c_str()));
        }
    }
}

void DbModel::editTime(const Time &time)
{
    if(hasTime(time.id()))
    {
        try
        {
            wxSQLite3Statement st = m_db.PrepareStatement("UPDATE time SET start=?,end=?,taskid=?"
                                                          " WHERE id=?");
            st.BindDateTime(1, time.start());
            st.BindDateTime(2, time.end());
            st.Bind(3, time.taskid());
            st.Bind(4, time.id());
            st.ExecuteUpdate();
            st.Finalize();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogError(wxString::Format("Error in editTime: %s", e.GetMessage().c_str()));
        }
    }
}

Time DbModel::getTime(int id)
{
    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM time WHERE id=%d", id));
    while(rs.NextRow())
    {
        return Time(rs.GetInt("id"), rs.GetDateTime("start"), rs.GetDateTime("end"), rs.GetInt("taskid"));
    }
    rs.Finalize();
    return Time();
}

bool DbModel::deleteTime(int id)
{
    try
    {
        m_db.ExecuteUpdate(wxString::Format("DELETE FROM time WHERE id=%d", id));
    }
    catch(const wxSQLite3Exception& e)
    {
        wxLogError(wxString::Format("Error in deleteTime: %s", e.GetMessage().c_str()));
        return false;
    }
    return true;
}

bool DbModel::hasTime(int id)
{
    return m_db.ExecuteScalar(wxString::Format("SELECT count(*) FROM time WHERE id=%d", id));
}

wxTimeSpan DbModel::timeSpanForTask(int taskid, TimeSum sum, bool subtasks)
{
    wxTimeSpan span;
    wxDateTime today = wxDateTime::Today();
    if(sum == TS_TODAY)
    {
        wxSQLite3ResultSet rs;
        if(taskid != -1) rs = m_db.ExecuteQuery(wxString::Format("SELECT start,end FROM time WHERE date(start)=date('%s') AND taskid=%d", today.FormatISODate(), taskid));
        else rs = m_db.ExecuteQuery(wxString::Format("SELECT start,end FROM time WHERE date(start)=date('%s')", today.FormatISODate()));
        while(rs.NextRow())
        {
            span += rs.GetDateTime(1).Subtract(rs.GetDateTime(0));
        }
        rs.Finalize();
    }
    if(sum == TS_WEEK)
    {
        wxDateTime::wxDateTime_t week = today.GetWeekOfYear(wxDateTime::Default_First);
        wxSQLite3ResultSet rs;
        if(taskid != -1) rs = m_db.ExecuteQuery(wxString::Format("SELECT start,end FROM time WHERE taskid=%d", taskid));
        else rs = m_db.ExecuteQuery("SELECT start,end FROM time");
        while(rs.NextRow())
        {
            if(rs.GetDateTime(0).GetWeekOfYear(wxDateTime::Default_First) == week) span += rs.GetDateTime(1).Subtract(rs.GetDateTime(0));
        }
        rs.Finalize();
    }
    if(sum == TS_MONTH)
    {
        wxSQLite3ResultSet rs;
        if(taskid != -1) rs = m_db.ExecuteQuery(wxString::Format("SELECT start,end FROM time WHERE strftime('%%Y-%%m',start)='%s' AND taskid=%d", today.Format("%Y-%m"), taskid));
        else rs = m_db.ExecuteQuery(wxString::Format("SELECT start,end FROM time WHERE strftime('%%Y-%%m',start)='%s'", today.Format("%Y-%m")));
        while(rs.NextRow())
        {
            span += rs.GetDateTime(1).Subtract(rs.GetDateTime(0));
        }
        rs.Finalize();
    }
    if(sum == TS_ALL)
    {
        wxSQLite3ResultSet rs;
        if(taskid != -1) rs = m_db.ExecuteQuery(wxString::Format("SELECT start,end FROM time WHERE taskid=%d", taskid));
        else rs = m_db.ExecuteQuery("SELECT start,end FROM time");
        while(rs.NextRow())
        {
            span += rs.GetDateTime(1).Subtract(rs.GetDateTime(0));
        }
        rs.Finalize();
    }
    if(subtasks && taskid != -1)
    {
        dxTaskArray stasks = subTasks(taskid);
        for(size_t j=0; j < stasks.GetCount(); j++)
        {
            span += timeSpanForTask(stasks[j].id(), sum, false);
        }
    }
    return span;
}

wxTimeSpan DbModel::timeSpanForReport(int taskid, const wxDateTime& start, const wxDateTime& end, bool subtasks)
{
    wxTimeSpan span;
    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT start,end FROM time WHERE taskid=%d", taskid));
    while(rs.NextRow())
    {
        if((rs.GetDateTime(0).IsSameDate(start) || rs.GetDateTime(0).IsLaterThan(start))
                && (rs.GetDateTime(0).IsSameDate(end) || rs.GetDateTime(0).IsEarlierThan(end)))
        {
            span += rs.GetDateTime(1).Subtract(rs.GetDateTime(0));
        }
    }
    rs.Finalize();
    if(subtasks)
    {
        dxTaskArray stasks = subTasks(taskid);
        for(size_t j=0; j < stasks.GetCount(); j++)
        {
            span += timeSpanForReport(stasks[j].id(), start, end, false);
        }
    }
    return span;
}

dxTimeArray DbModel::timesByTaks(int taskid, TimeSum sum, bool subtasks)
{
    dxTimeArray times;
    wxDateTime today = wxDateTime::Today();
    if(sum == TS_TODAY)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM time WHERE date(start)=date('%s') AND taskid=%d", today.FormatISODate(), taskid));
        while(rs.NextRow())
        {
            times.Add(Time(rs.GetInt("id"), rs.GetDateTime("start"), rs.GetDateTime("end"), taskid));
        }
        rs.Finalize();
        if(subtasks)
        {
            dxTaskArray stasks = subTasks(taskid);
            for(size_t i=0; i < stasks.GetCount(); i++)
            {
                dxTimeArray stimes = timesByTaks(stasks[i].id(), TS_TODAY);
                for(size_t j=0; j < stimes.GetCount(); j++)
                {
                    times.Add(stimes[j]);
                }
            }
        }
    }
    if(sum == TS_WEEK)
    {
        wxDateTime::wxDateTime_t week = today.GetWeekOfYear(wxDateTime::Default_First);
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM time WHERE taskid=%d", taskid));
        while(rs.NextRow())
        {
            if(rs.GetDateTime("start").GetWeekOfYear(wxDateTime::Default_First) == week) times.Add(Time(rs.GetInt("id"), rs.GetDateTime("start"), rs.GetDateTime("end"), taskid));
        }
        rs.Finalize();
        if(subtasks)
        {
            dxTaskArray stasks = subTasks(taskid);
            for(size_t i=0; i < stasks.GetCount(); i++)
            {
                dxTimeArray stimes = timesByTaks(stasks[i].id(), TS_WEEK);
                for(size_t j=0; j < stimes.GetCount(); j++)
                {
                    times.Add(stimes[j]);
                }
            }
        }
    }
    if(sum == TS_MONTH)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM time WHERE strftime('%%Y-%%m',start)='%s' AND taskid=%d", today.Format("%Y-%m"), taskid));
        while(rs.NextRow())
        {
            times.Add(Time(rs.GetInt("id"), rs.GetDateTime("start"), rs.GetDateTime("end"), taskid));
        }
        rs.Finalize();
        if(subtasks)
        {
            dxTaskArray stasks = subTasks(taskid);
            for(size_t i=0; i < stasks.GetCount(); i++)
            {
                dxTimeArray stimes = timesByTaks(stasks[i].id(), TS_MONTH);
                for(size_t j=0; j < stimes.GetCount(); j++)
                {
                    times.Add(stimes[j]);
                }
            }
        }
    }
    if(sum == TS_ALL)
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM time WHERE taskid=%d", taskid));
        while(rs.NextRow())
        {
            times.Add(Time(rs.GetInt("id"), rs.GetDateTime("start"), rs.GetDateTime("end"), taskid));
        }
        rs.Finalize();
        if(subtasks)
        {
            dxTaskArray stasks = subTasks(taskid);
            for(size_t i=0; i < stasks.GetCount(); i++)
            {
                dxTimeArray stimes = timesByTaks(stasks[i].id(), TS_ALL);
                for(size_t j=0; j < stimes.GetCount(); j++)
                {
                    times.Add(stimes[j]);
                }
            }
        }
    }
    return times;
}

dxTimeArray DbModel::timesForReport(int taskid, const wxDateTime& start, const wxDateTime& end)
{
    dxTimeArray times;
    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT * FROM time WHERE taskid=%d", taskid));
    while(rs.NextRow())
    {
        if((rs.GetDateTime("start").IsSameDate(start) || rs.GetDateTime("start").IsLaterThan(start))
                && (rs.GetDateTime("start").IsSameDate(end) || rs.GetDateTime("start").IsEarlierThan(end)))
        {
            times.Add(Time(rs.GetInt("id"), rs.GetDateTime("start"), rs.GetDateTime("end"), taskid));
        }
    }
    rs.Finalize();
    return times;
}

int DbModel::getMinutes(int id)
{
    int minutes = 0;
    if(hasTime(id))
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT start,end FROM time WHERE id=%d", id));
        while(rs.NextRow())
        {
            minutes += rs.GetDateTime(1).Subtract(rs.GetDateTime(0)).GetMinutes();
        }
        rs.Finalize();
    }
    return minutes;
}

wxString DbModel::getTimeString(int id)
{
    if(hasTime(id))
    {
        wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT start,end FROM time WHERE id=%d", id));
        while(rs.NextRow())
        {
            return rs.GetDateTime(1).Subtract(rs.GetDateTime(0)).Format("%H:%M");
        }
        rs.Finalize();
    }
    return wxEmptyString;
}

bool DbModel::timeExists(const wxDateTime& start, const wxDateTime& end, int timeid)
{
    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT start,end,id FROM time");
    while(rs.NextRow())
    {
        if(rs.GetInt(2) != timeid)
        {
            if(start.IsBetween(rs.GetDateTime(0), rs.GetDateTime(1)) || end.IsBetween(rs.GetDateTime(0), rs.GetDateTime(1)))
                return true;
            if(rs.GetDateTime(0).IsBetween(start, end) && rs.GetDateTime(1).IsBetween(start, end))
                return true;
        }
    }
    rs.Finalize();
    return false;
}

void DbModel::setEndTime(int id, const wxDateTime& end)
{
    if(hasTime(id))
    {
        try
        {
            wxSQLite3Statement st = m_db.PrepareStatement("UPDATE time SET end=?"
                                                          " WHERE id=?");
            st.BindDateTime(1, end);
            st.Bind(2, id);
            st.ExecuteUpdate();
            st.Finalize();
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogError(wxString::Format("Error in setEndTime: %s", e.GetMessage().c_str()));
        }
    }
}

void DbModel::sortTask(dxTaskArray& arr, DbModel::TaskSort sort, bool asc)
{
    switch(sort)
    {
    case TAS_TREE:
    {
        if(asc) arr.Sort(dxSortTaskByTreeAsc);
        else arr.Sort(dxSortTaskByTreeDsc);
        return;
    }
    default: return;
    }
}

void DbModel::sortTime(dxTimeArray &arr, DbModel::TimeSort sort, bool asc)
{
    switch(sort)
    {
    case TIS_START:
    {
        if(asc) arr.Sort(dxSortTimeByStartAsc);
        else arr.Sort(dxSortTimeByStartDsc);
        return;
    }
    case TIS_END:
    {
        if(asc) arr.Sort(dxSortTimeByEndAsc);
        else arr.Sort(dxSortTimeByEndDsc);
        return;
    }
    case TIS_TIME:
    {
        if(asc) arr.Sort(dxSortTimeByTimeAsc);
        else arr.Sort(dxSortTimeByTimeDsc);
        return;
    }
    case TIS_TASK:
    {
        if(asc) arr.Sort(dxSortTimeByTaskAsc);
        else arr.Sort(dxSortTimeByTaskDsc);
        return;
    }
    default: return;
    }
}

void DbModel::verifyTimes()
{
    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT taskid FROM time");
    while(rs.NextRow())
    {
        if(!hasTableId("task", "id", rs.GetInt(0)))
        {
            tryExecuteUpdate(wxString::Format("DELETE FROM time WHERE taskid=%d", rs.GetInt(0)));
        }
    }
    rs.Finalize();
}

wxDateTime DbModel::oldestDate()
{
    wxDateTime date = wxDateTime::Now();
    wxSQLite3ResultSet rs = m_db.ExecuteQuery("SELECT start FROM time");
    while(rs.NextRow())
    {
        if(rs.GetDateTime(0).IsEarlierThan(date))
        {
            date = rs.GetDateTime(0);
        }
    }
    rs.Finalize();
    return date;
}

bool DbModel::hasTableId(const wxString &table, const wxString &idcolumn, int id)
{
        if(!m_db.TableExists(table)) return false;
        else
        {
            wxArrayString arr = tableColumns(table);
            if(!dxutils::hasString(arr, idcolumn)) return false;
            else
            {
                wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("SELECT %s FROM %s", idcolumn, table));
                while(rs.NextRow())
                {
                    if(id == rs.GetInt(0))
                    {
                        rs.Finalize();
                        return true;
                    }
                }
                rs.Finalize();
            }
        }
    return false;
}

wxArrayString DbModel::tableColumns(const wxString &name)
{
    wxArrayString arr;
    wxSQLite3ResultSet rs = m_db.ExecuteQuery(wxString::Format("PRAGMA table_info(%s)", name));
    while(rs.NextRow())
    {
        arr.Add(rs.GetString(1));
    }
    rs.Finalize();
    return arr;
}

void DbModel::tryExecuteUpdate(const wxString &sql)
{
    if(m_db.IsOpen())
    {
        try
        {
            m_db.ExecuteUpdate(sql);
        }
        catch(const wxSQLite3Exception& e)
        {
            wxLogDebug(wxString::Format("Error in tryExecuteUpdate \"%s\": %s", sql, e.GetMessage()));
        }
    }
    else
    {
        wxLogDebug("Database isn't open");
    }
}
